#include "StdAfx.h"
#include "SoundGen.h"

SoundGen::SoundGen(int samplesPerSecond)
{
	SamplesPerSecond = samplesPerSecond;

	pFirstTimeSpace=NULL;
	pLastTimeSpace=NULL;

	BufferLength=0;
	BufferValid=true;
	Buffer=NULL;
}

SoundGen::~SoundGen(void)
{
	Clear();
}

int SoundGen::AddTimeSpace(int milliseconds)
{
	TimeSpace* space = new TimeSpace;

	space->Samples=(int)((float)milliseconds / 1000.0f * SamplesPerSecond);
	BufferLength+=space->Samples;

	space->pFirstSineWave=NULL;
	space->pLastSineWave=NULL;
	space->pNext=NULL;

	if(pFirstTimeSpace==NULL)
	{
		space->Index=1;
		pFirstTimeSpace=space;
		pLastTimeSpace=space;
	}
	else
	{
		space->Index=pLastTimeSpace->Index+1;
		pLastTimeSpace->pNext=space;
		pLastTimeSpace=space;
	}

	BufferValid=false;
	return space->Index;
}

void SoundGen::AddSineWaveToTimeSpace(int spaceIndex, float frequency, float amplitude)
{
	//Najit prostor
	TimeSpace* pActualSpace = pFirstTimeSpace;
	while(pActualSpace!=NULL && pActualSpace->Index!=spaceIndex)
		pActualSpace=pActualSpace->pNext;

	if(pActualSpace==NULL)
		return;

	//Pridat vlnu
	SineWave* wave=new SineWave;
	wave->Amplitude=amplitude;
	wave->Frequency=frequency;
	wave->pNext=NULL;

	if(pActualSpace->pFirstSineWave==NULL)
	{
		pActualSpace->pFirstSineWave=wave;
		pActualSpace->pLastSineWave=wave;
	}
	else
	{
		pActualSpace->pLastSineWave->pNext=wave;
		pActualSpace->pLastSineWave=wave;
	}

	BufferValid=false;
}

void SoundGen::UpdateBuffer()
{
	if(BufferValid)
		return;

	//Smazat stara data
	if(Buffer!=NULL)
	{
		delete[] Buffer;
		Buffer=NULL;
	}

	//Vytvorit jednotlive zvuky
	Buffer=new BYTE[BufferLength];
	TimeSpace* pActualSpace=pFirstTimeSpace;
	int position=0;
	while(pActualSpace!=NULL)
	{
		position+=GenerateSpace(pActualSpace, Buffer + position);
		pActualSpace=pActualSpace->pNext;
	}

	BufferValid=true;
}

int SoundGen::GenerateSpace(TimeSpace* pSpace, BYTE* targetBuffer)
{
	double t;
	double totalValue;
	SineWave* pActualWave;
	int wavesCount;

	for(int i=0;i<pSpace->Samples;i++)
	{
		t = (double)i / (double)SamplesPerSecond;
		totalValue=0;
		wavesCount=0;

		//Projit a secist vlny
		pActualWave=pSpace->pFirstSineWave;
		while(pActualWave!=NULL)
		{
			totalValue += sin(2*PI*pActualWave->Frequency * t) * pActualWave->Amplitude;

			pActualWave=pActualWave->pNext;
			wavesCount++;
		}

		targetBuffer[i] = (int)((totalValue / (double)wavesCount) * 127.0 + 127);
	}

	return pSpace->Samples;
}

int SoundGen::GetBufferLength()
{
	return BufferLength;
}

const BYTE* SoundGen::GetBuffer()
{
	UpdateBuffer();
	return Buffer;
}

void SoundGen::Clear()
{
	TimeSpace* pActualSpace = pFirstTimeSpace;
	SineWave* pActualWave;

	while(pActualSpace!=NULL)
	{
		pActualWave = pActualSpace->pFirstSineWave;
		while(pActualWave!=NULL)
		{
			pActualSpace->pFirstSineWave=pActualWave->pNext;
			delete pActualWave;
			pActualWave=pActualSpace->pFirstSineWave;
		}

		pFirstTimeSpace = pActualSpace->pNext;
		delete pActualSpace;
		pActualSpace = pFirstTimeSpace;
	}

	pFirstTimeSpace=NULL;
	pLastTimeSpace=NULL;

	if(Buffer!=NULL)
	{
		delete[] Buffer;
		Buffer=NULL;
	}
	BufferValid=true;
	BufferLength=0;
}